local L = BAFK_locale()
local addonName = "Broker_AFK"
local LaddonName = L[addonName]
Broker_AFK = {}
local addon = Broker_AFK
local _G = getfenv(0)
BAFK_svnrev = {}
BAFK_svnrev["Broker_AFK.lua"] = tonumber(("$Revision: 3 $"):match("%d+"))
local defaults = { 
  DebugEnabled = false,
  timeaway = true,
}
local settings = defaults
local LDB, LDBo
local status_avail = 1
local status_afk   = 2
local status_dnd   = 3
local current_status = nil
local current_status_time = nil
local elapsed_time = 0
local timeaway = ""
local status_icon = {
  [status_avail] = FRIENDS_TEXTURE_ONLINE,
  [status_afk]   = FRIENDS_TEXTURE_AFK,
  [status_dnd]   = FRIENDS_TEXTURE_DND,
}
local status_text = {
  [status_avail] = FRIENDS_LIST_AVAILABLE,
  [status_afk]   = FRIENDS_LIST_AWAY,
  [status_dnd]   = FRIENDS_LIST_BUSY,
}
local SEC_TO_MINUTE_FACTOR = 1/60;
local SEC_TO_HOUR_FACTOR = SEC_TO_MINUTE_FACTOR*SEC_TO_MINUTE_FACTOR;

local function chatMsg(msg)
     DEFAULT_CHAT_FRAME:AddMessage(LaddonName..": "..msg)
end
local function debug(msg)
  if settings.DebugEnabled then
     chatMsg(msg)
  end
end

local frame = CreateFrame("Button", addonName.."HiddenFrame", UIParent)
frame:RegisterEvent("ADDON_LOADED");
frame:RegisterEvent("PLAYER_FLAGS_CHANGED");
frame:RegisterEvent("PLAYER_ENTERING_WORLD");

local function OnEvent(frame, event, name, ...)
  if event == "ADDON_LOADED" and name == addonName then
     debug("ADDON_LOADED: "..name)
     addon:setupLDB()
     addon:SetupVersion()
     Broker_AFK = Broker_AFK or {}
     if not Broker_AFK.settings then
         Broker_AFK.settings = {}
         for k,v in pairs(defaults) do
           Broker_AFK.settings[k] = v
         end
     end
     settings = Broker_AFK.settings
     settings.loaded = true
  elseif event == "ADDON_LOADED" then
     debug("ADDON_LOADED: "..name)
     addon:setupLDB()
  elseif event == "PLAYER_FLAGS_CHANGED" or event == "PLAYER_ENTERING_WORLD" then
     addon:Update()
  end
end
frame:SetScript("OnEvent", OnEvent);

SLASH_BROKERAFK1 = L["/bafk"]
SlashCmdList["BROKERAFK"] = function(msg)
        local cmd = msg:lower()
        if cmd == L["debug"] then
          chatMsg(L["debug toggled"])
          settings.DebugEnabled = not settings.DebugEnabled
        else
          chatMsg(LaddonName.." "..addon.version)
          chatMsg(SLASH_BROKERAFK1.." [ "..L["minimap"].." | "..L["debug"].." ]")
        end
end

function addon:SetupVersion()
   local svnrev = 0
   local T_svnrev = BAFK_svnrev
   T_svnrev["X-Build"] = tonumber((GetAddOnMetadata(addonName, "X-Build") or ""):match("%d+"))
   T_svnrev["X-Revision"] = tonumber((GetAddOnMetadata(addonName, "X-Revision") or ""):match("%d+"))
   for _,v in pairs(T_svnrev) do -- determine highest file revision
     if v and v > svnrev then
       svnrev = v
     end
   end
   addon.revision = svnrev

   T_svnrev["X-Curse-Packaged-Version"] = GetAddOnMetadata(addonName, "X-Curse-Packaged-Version")
   T_svnrev["Version"] = GetAddOnMetadata(addonName, "Version")
   addon.version = T_svnrev["X-Curse-Packaged-Version"] or T_svnrev["Version"] or "@"
   if string.find(addon.version, "@") then -- dev copy uses "@.project-version.@"
      addon.version = "r"..svnrev
   end
end

local function OnUpdate(f,el) addon:Update(el) end

function addon:Update(elapsed)
  if elapsed then -- optimize away unnecessary updates
     elapsed_time = elapsed_time + elapsed
     if elapsed_time < 0.25 then
       return
     end
  else
     elapsed_time = 0.0
  end

  local new_status = status_avail
  if (IsChatAFK()) then
    new_status = status_afk
  elseif (IsChatDND()) then
    new_status = status_dnd
  end

  if new_status ~= current_status then -- change in status
    current_status_time = GetTime()
    current_status = new_status
  end

  if current_status == status_avail then
    PlayerName:SetText(UnitName("player"))
  else
    PlayerName:SetText("|T"..status_icon[current_status]..":0|t "..
                       UnitName("player").." <"..status_text[current_status]..">")
  end
  if settings.timeaway and current_status == status_afk then
     frame:SetScript("OnUpdate", OnUpdate);
     local total_time = GetTime() - current_status_time;
     local hour = min(floor(total_time*SEC_TO_HOUR_FACTOR), 99);
     local minute = mod(total_time*SEC_TO_MINUTE_FACTOR, 60);
     local second = mod(total_time, 60);
     if hour >= 1 then
       timeaway = string.format(" %d:%02d:%02d", hour, minute, second)
     elseif minute >= 1 then
       timeaway = string.format(" %d:%02d", minute, second)
     else
       timeaway = string.format(" :%02d",second)
     end
     if addon.tooltip and addon.tooltipOwner == addon.tooltip:GetOwner() then
       BAFK_OnTooltipShow(addon.tooltip)
     end
  else
     timeaway = ""
     frame:SetScript("OnUpdate", nil);
  end

  if LDBo then
    LDBo.text = status_text[current_status]..timeaway
    LDBo.icon = status_icon[current_status]
  end
end

function BAFK_OnClick(frame, button)
   if addon.tooltip and addon.tooltip:IsVisible() then
      addon.tooltip:Hide()
      addon.tooltip = nil
   end
   if ( IsShiftKeyDown() ) then
	debug("shift-click "..button)
   elseif button == "LeftButton" then
        debug("status menu")
        ToggleDropDownMenu(1, nil, FriendsFrameStatusDropDown, "cursor", 0, 0)
   elseif button == "RightButton" then
        debug("config menu")
        ToggleDropDownMenu(1, nil, addon.DropDownMenu, "cursor", 0, 0)
   else
	debug("click "..button)
   end
end

function BAFK_OnTooltipShow(tooltip)
     addon.tooltip = tooltip
     addon.tooltipOwner = tooltip:GetOwner()
     if tooltip and tooltip.AddLine then
         tooltip:SetText(LaddonName.." "..addon.version)
         tooltip:AddLine(string.format(FRIENDS_LIST_STATUS_TOOLTIP, status_text[current_status])..timeaway)
         tooltip:AddLine("|cffff8040"..L["Left Click"].."|r "..L["to set status"])
         tooltip:AddLine("|cffff8040"..L["Right Click"].."|r "..L["for config"])
         tooltip:Show()
     end
end

function addon:setupLDB()
  if LDB then
    return
  end
  if AceLibrary and AceLibrary:HasInstance("LibDataBroker-1.1") then
    LDB = AceLibrary("LibDataBroker-1.1")
  elseif LibStub then
    LDB = LibStub:GetLibrary("LibDataBroker-1.1",true)
  end
  if LDB then
    LDBo = LDB:NewDataObject(LaddonName, {
        type = "data source",
        label = LaddonName,
	text = LaddonName,
        OnClick = BAFK_OnClick,
        OnTooltipShow = BAFK_OnTooltipShow,
        OnTooltipHide = function() addon.tooltip = nil end,
     })
     debug("LDB created!")
  end 
  addon:Update()
end

addon.DropDownMenu = CreateFrame("Frame", addonName.."_DropDownMenu")
addon.DropDownMenu.displayMode = "MENU"
addon.DropDownMenu.onHide = function(...)
        MenuParent = nil
        MenuItem = nil
end

local menuinfo = {}
addon.DropDownMenu.initialize = function(self, level)
        if not level then return end
        wipe(menuinfo)
        if level == 1 then
                -- Create the title of the menu
                menuinfo.isTitle = 1
                menuinfo.text = LaddonName.." "..addon.version
                menuinfo.notCheckable = 1
                UIDropDownMenu_AddButton(menuinfo, level)

                menuinfo.disabled     = nil
                menuinfo.isTitle      = nil
                menuinfo.notCheckable = nil

                local options = { "foo" }
                for _,key in pairs(options) do
                        menuinfo.text = L[key] 
                        menuinfo.arg1 = key
                        menuinfo.func = function(button, arg1)
                               settings[key] = not settings[key]
        		       chatMsg(L[arg1].." "..(settings[key] and VIDEO_OPTIONS_ENABLED or VIDEO_OPTIONS_DISABLED))
                        end
                        menuinfo.checked = (settings[key])
                        UIDropDownMenu_AddButton(menuinfo, level)
                end

                -- Close menu item
                menuinfo.text         = CLOSE
                menuinfo.func         = function() CloseDropDownMenus() end
                menuinfo.checked      = nil
                menuinfo.notCheckable = 1
                UIDropDownMenu_AddButton(menuinfo, level)
        end
end
